@@ -6,6 +6,7 @@
 #include <private/qplatformmediacapture_p.h>
 #include "avfcamerautility_p.h"
 #include "qavfhelpers_p.h"
+#include "qavfsamplebufferdelegate_p.h"
 #include <qvideosink.h>
 #include <private/qrhi_p.h>
 #define AVMediaType XAVMediaType
@@ -17,147 +18,16 @@ extern "C" {
 }
 #undef AVMediaType
 
-
-
-#import <AVFoundation/AVFoundation.h>
-#include <CoreVideo/CoreVideo.h>
-
-static void releaseHwFrame(void */*opaque*/, uint8_t *data)
-{
-    CVPixelBufferRelease(CVPixelBufferRef(data));
-}
-
-// Make sure this is compatible with the layout used in ffmpeg's hwcontext_videotoolbox
-static AVFrame *allocHWFrame(AVBufferRef *hwContext, const CVPixelBufferRef &pixbuf)
-{
-    AVHWFramesContext *ctx = (AVHWFramesContext*)hwContext->data;
-    AVFrame *frame = av_frame_alloc();
-    frame->hw_frames_ctx = av_buffer_ref(hwContext);
-    frame->extended_data = frame->data;
-
-    frame->buf[0] = av_buffer_create((uint8_t *)pixbuf, 1, releaseHwFrame, NULL, 0);
-    frame->data[3] = (uint8_t *)pixbuf;
-    CVPixelBufferRetain(pixbuf);
-    frame->width  = ctx->width;
-    frame->height = ctx->height;
-    frame->format = AV_PIX_FMT_VIDEOTOOLBOX;
-    if (frame->width != (int)CVPixelBufferGetWidth(pixbuf) ||
-        frame->height != (int)CVPixelBufferGetHeight(pixbuf)) {
-        // This can happen while changing camera format
-        av_frame_free(&frame);
-        return nullptr;
-    }
-    return frame;
-}
-
 static AVAuthorizationStatus m_cameraAuthorizationStatus = AVAuthorizationStatusNotDetermined;
 
-@interface QAVFSampleBufferDelegate : NSObject <AVCaptureVideoDataOutputSampleBufferDelegate>
-
-- (QAVFSampleBufferDelegate *) initWithCamera:(QAVFCamera *)renderer;
-
-- (void) captureOutput:(AVCaptureOutput *)captureOutput
-         didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
-         fromConnection:(AVCaptureConnection *)connection;
-
-- (void) setHWAccel:(std::unique_ptr<QFFmpeg::HWAccel> &&)accel;
-
-@end
-
-@implementation QAVFSampleBufferDelegate
-{
-@private
-    QAVFCamera *m_camera;
-    AVBufferRef *hwFramesContext;
-    std::unique_ptr<QFFmpeg::HWAccel> m_accel;
-    qint64 startTime;
-    qint64 baseTime;
-}
-
-- (QAVFSampleBufferDelegate *) initWithCamera:(QAVFCamera *)renderer
-{
-    if (!(self = [super init]))
-        return nil;
-
-    m_camera = renderer;
-    hwFramesContext = nullptr;
-    startTime = 0;
-    baseTime = 0;
-    return self;
-}
-
-- (void)captureOutput:(AVCaptureOutput *)captureOutput
-         didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer
-         fromConnection:(AVCaptureConnection *)connection
-{
-    Q_UNUSED(connection);
-    Q_UNUSED(captureOutput);
-
-    // NB: on iOS captureOutput/connection can be nil (when recording a video -
-    // avfmediaassetwriter).
-
-    CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
-
-    CMTime time = CMSampleBufferGetPresentationTimeStamp(sampleBuffer);
-    qint64 frameTime = time.timescale ? time.value*1000/time.timescale : 0;
-    if (baseTime == 0) {
-        // drop the first frame to get a valid frame start time
-        baseTime = frameTime;
-        startTime = 0;
-        return;
-    }
-
-    if (!m_accel)
-        return;
-
-    AVFrame *avFrame = allocHWFrame(m_accel->hwFramesContextAsBuffer(), imageBuffer);
-    if (!avFrame)
-        return;
-
-#ifdef USE_SW_FRAMES
-    auto *swFrame = av_frame_alloc();
-    /* retrieve data from GPU to CPU */
-    int ret = av_hwframe_transfer_data(swFrame, avFrame, 0);
-    if (ret < 0) {
-        qWarning() << "Error transferring the data to system memory\n";
-        av_frame_unref(swFrame);
-    } else {
-        av_frame_unref(avFrame);
-        avFrame = swFrame;
-    }
-#endif
-
-    QVideoFrameFormat format = QAVFHelpers::videoFormatForImageBuffer(imageBuffer);
-    if (!format.isValid()) {
-        av_frame_unref(avFrame);
-        return;
-    }
-
-    avFrame->pts = startTime;
-
-    QFFmpegVideoBuffer *buffer = new QFFmpegVideoBuffer(avFrame);
-    QVideoFrame frame(buffer, format);
-    frame.setStartTime(startTime);
-    frame.setEndTime(frameTime);
-    startTime = frameTime;
-
-    m_camera->syncHandleFrame(frame);
-}
-
-- (void) setHWAccel:(std::unique_ptr<QFFmpeg::HWAccel> &&)accel
-{
-    m_accel = std::move(accel);
-}
-
-@end
-
 QT_BEGIN_NAMESPACE
 
 QAVFCamera::QAVFCamera(QCamera *parent)
     : QAVFCameraBase(parent)
 {
     m_captureSession = [[AVCaptureSession alloc] init];
-    m_sampleBufferDelegate = [[QAVFSampleBufferDelegate alloc] initWithCamera:this];
+    m_sampleBufferDelegate = [[QAVFSampleBufferDelegate alloc]
+            initWithFrameHandler:[this](const QVideoFrame &frame) { syncHandleFrame(frame); }];
 }
 
 QAVFCamera::~QAVFCamera()
@@ -222,7 +92,7 @@ void QAVFCamera::updateVideoInput()
     attachVideoInputDevice();
 
     if (!m_videoDataOutput) {
-        m_videoDataOutput = [[[AVCaptureVideoDataOutput alloc] init] autorelease];
+        m_videoDataOutput = [[AVCaptureVideoDataOutput alloc] init];
 
         // Configure video output
         m_delegateQueue = dispatch_queue_create("vf_queue", nullptr);
@@ -363,53 +233,67 @@ void QAVFCamera::updateCameraFormat()
     if (!captureDevice)
         return;
 
-    uint avPixelFormat = 0;
+    std::uint32_t cvPixelFormat = 0;
     AVCaptureDeviceFormat *newFormat = qt_convert_to_capture_device_format(captureDevice, m_cameraFormat);
     if (newFormat) {
         qt_set_active_format(captureDevice, newFormat, false);
-        avPixelFormat = setPixelFormat(m_cameraFormat.pixelFormat());
+        cvPixelFormat = setPixelFormat(m_cameraFormat.pixelFormat());
+    }
+
+    const AVPixelFormat avPixelFormat = av_map_videotoolbox_format_to_pixfmt(cvPixelFormat);
+
+    std::unique_ptr<QFFmpeg::HWAccel> hwAccel;
+
+    if (avPixelFormat == AV_PIX_FMT_NONE) {
+        auto formatDescIt =
+                std::make_reverse_iterator(reinterpret_cast<const char *>(&cvPixelFormat));
+        qWarning() << "Videotoolbox desn't support cvPixelFormat:" << cvPixelFormat
+                   << std::string(formatDescIt - 4, formatDescIt)
+                   << " Camera pix format:" << m_cameraFormat.pixelFormat();
+    } else {
+        hwAccel = QFFmpeg::HWAccel::create(AV_HWDEVICE_TYPE_VIDEOTOOLBOX);
     }
 
-    auto hwAccel = QFFmpeg::HWAccel::create(AV_HWDEVICE_TYPE_VIDEOTOOLBOX);
     if (hwAccel) {
-        hwAccel->createFramesContext(av_map_videotoolbox_format_to_pixfmt(avPixelFormat),
-                                     m_cameraFormat.resolution());
+        hwAccel->createFramesContext(avPixelFormat, m_cameraFormat.resolution());
         hwPixelFormat = hwAccel->hwFormat();
     } else {
         hwPixelFormat = AV_PIX_FMT_NONE;
     }
     [m_sampleBufferDelegate setHWAccel:std::move(hwAccel)];
+    [m_sampleBufferDelegate setVideoFormatFrameRate:m_cameraFormat.maxFrameRate()];
 }
 
-uint QAVFCamera::setPixelFormat(const QVideoFrameFormat::PixelFormat pixelFormat)
+std::uint32_t QAVFCamera::setPixelFormat(const QVideoFrameFormat::PixelFormat pixelFormat)
 {
     // Default to 32BGRA pixel formats on the viewfinder, in case the requested
     // format can't be used (shouldn't happen unless the developers sets a wrong camera
     // format on the camera).
-    unsigned avPixelFormat = kCVPixelFormatType_32BGRA;
-    if (!QAVFHelpers::toCVPixelFormat(pixelFormat, avPixelFormat))
+    std::uint32_t cvPixelFormat = kCVPixelFormatType_32BGRA;
+    if (!QAVFHelpers::toCVPixelFormat(pixelFormat, cvPixelFormat))
         qWarning() << "QCamera::setCameraFormat: couldn't convert requested pixel format, using ARGB32";
 
     bool isSupported = false;
     NSArray *supportedPixelFormats = m_videoDataOutput.availableVideoCVPixelFormatTypes;
     for (NSNumber *currentPixelFormat in supportedPixelFormats)
     {
-        if ([currentPixelFormat unsignedIntValue] == avPixelFormat) {
+        if ([currentPixelFormat unsignedIntValue] == cvPixelFormat) {
             isSupported = true;
             break;
         }
     }
 
     if (isSupported) {
-        NSDictionary* outputSettings = @{
-            (NSString *)kCVPixelBufferPixelFormatTypeKey: [NSNumber numberWithUnsignedInt:avPixelFormat],
-            (NSString *)kCVPixelBufferMetalCompatibilityKey: @true
+        NSDictionary *outputSettings = @{
+            (NSString *)
+            kCVPixelBufferPixelFormatTypeKey : [NSNumber numberWithUnsignedInt:cvPixelFormat],
+            (NSString *)kCVPixelBufferMetalCompatibilityKey : @true
         };
         m_videoDataOutput.videoSettings = outputSettings;
     } else {
         qWarning() << "QCamera::setCameraFormat: requested pixel format not supported. Did you use a camera format from another camera?";
     }
-    return avPixelFormat;
+    return cvPixelFormat;
 }
 
 void QAVFCamera::syncHandleFrame(const QVideoFrame &frame)
